"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var moment = require("moment");
var Connector_1 = require("../Connector");
var translator_1 = require("../translator");
var locale = 'en';
var messages = immutable_1.fromJS({
    en: {
        'Homepage headline': 'Super headline',
        'Interpolated string': 'This was interpolated: %{foo}',
        homepage: {
            'Homepage headline': 'Super headline under scope',
        },
        deep: {
            scopedTransInterpolation: '%{foo} (again)',
            scope: {
                'Homepage headline': 'Super headline under deep scope',
            },
        },
        formats: {
            date: {
                default: {
                    format: 'D.M.YYYY',
                },
                short: {
                    format: 'D.M.YYYY',
                },
                long: {
                    format: 'H:m:s D.M.YYYY',
                },
            },
            number: {
                default: {
                    precision: 10,
                },
                currency: {
                    unit: '$',
                },
                percentage: {
                    template: '%n %',
                },
            },
        },
    },
    es: {
        'Homepage headline': 'Super titular',
    },
});
var flatMapMessages = immutable_1.fromJS({
    'en.views.homepage.title': 'Welcome!',
    'en.views.homepage.subsection.heading': 'First title',
    'en.views.homepage.subsection.text': 'Lorem ipsum dolor',
    'es.views.homepage.title': 'Bueno!',
});
var hybridMapMessages = immutable_1.fromJS({
    en: {
        'views.homepage.title': 'Welcome!',
        'views.homepage.subsection.heading': 'First title',
        'views.homepage.subsection.text': 'Lorem ipsum dolor',
    },
    es: {
        'views.homepage.title': 'Bueno!',
    },
});
it('should return default text', function () {
    expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Not translated')).toEqual('Not translated');
});
it('should return undefined when disabledDefault', function () {
    expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Not translated', { disableDefault: true })).toEqual(undefined);
});
it('should return text from messages', function () {
    expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Homepage headline')).toEqual('Super headline');
});
describe('Using array as keys for fallbacking', function () {
    it('should return text from first found key', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg(['deep.scope.Homepage headline', 'homepage.Homepage headline'])).toEqual('Super headline under deep scope');
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg(['deep.scope.unknown', 'homepage.Homepage headline'])).toEqual('Super headline under scope');
    });
    it('should return default text from first not found key', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg(['deep.scope.unknown', 'homepage.unknown'])).toEqual('deep.scope.unknown');
    });
});
describe('Using scope', function () {
    it('should return text from messages with scope', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Homepage headline', { scope: 'homepage' })).toEqual('Super headline under scope');
    });
    it('should return text from messages with nested key path', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('homepage.Homepage headline')).toEqual('Super headline under scope');
    });
    it('should return text from messages with nested array key', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg(['homepage', 'Homepage headline'])).toEqual('Super headline under scope');
    });
    it('should return default text from messages with unknown scope', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Homepage headline', { scope: 'unknown' })).toEqual('Homepage headline');
    });
    it('should return text from messages with deep scope', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Homepage headline', { scope: 'deep.scope' })).toEqual('Super headline under deep scope');
    });
    it('should return object containing all messages under given key', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('deep')).toEqual(messages.getIn(['en', 'deep']).toJS());
    });
});
describe('Fallbacking locales', function () {
    it('should return text in es from messages', function () {
        expect(new translator_1.Translator({ messages: messages, locale: 'es', fallbackLocale: 'en' }).msg('Homepage headline')).toEqual('Super titular');
    });
    it('should return fallback text in default en from messages', function () {
        expect(new translator_1.Translator({ messages: messages, locale: 'es', fallbackLocale: 'en' }).msg('Homepage headline', { scope: 'homepage' })).toEqual('Super headline under scope');
    });
    it('should return default text when not found in locale and defaultLocale', function () {
        expect(new translator_1.Translator({ messages: messages, locale: 'es', fallbackLocale: 'en' }).msg('Not translated')).toEqual('Not translated');
    });
});
describe('formatDate', function () {
    var date = moment('2017-02-28T12:58:20.006');
    it('returns formatted date with only defaults', function () {
        expect(new translator_1.Translator({ locale: locale }).formatDate(date)).toEqual('28.2.2017');
    });
    it('returns formatted date for existing options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatDate(date)).toEqual('28.2.2017');
    });
    it('returns formatted date for nonexisting options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: 'es' }).formatDate(date)).toEqual('28.2.2017');
    });
    it('returns formatted date with custom format', function () {
        expect(new translator_1.Translator({ messages: messages, locale: 'es' }).formatDate(date, 'YYYY')).toEqual('2017');
    });
    it('returns formatted date with keyword format', function () {
        expect(new translator_1.Translator({ messages: messages, locale: 'en' }).formatDate(date, 'short')).toEqual('28.2.2017');
        expect(new translator_1.Translator({ messages: messages, locale: 'en' }).formatDate(date, 'long')).toEqual('12:58:20 28.2.2017');
    });
});
describe('formatNumber', function () {
    it('returns formatted number with only defaults', function () {
        expect(new translator_1.Translator({ locale: locale }).formatNumber(123456.789)).toEqual('123,456.789');
    });
    it('returns formatted number without any options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatNumber(123456.789)).toEqual('123,456.789');
    });
    it('returns formatted number with options in messages', function () {
        expect(new translator_1.Translator({ messages: messages.setIn(['en', 'formats', 'number', 'default', 'template'], '%n %'), locale: locale }).formatNumber(123456.789)).toEqual('123,456.789 %');
    });
    it('returns formatted number with currency option', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatNumber(123456.789, 'currency')).toEqual('123,456.789 $');
    });
    it('returns formatted number with percentage option', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatNumber(123456.789, 'percentage')).toEqual('123,456.789 %');
    });
    it('returns formatted number with override options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatNumber(123456.789, { template: '$ %n' })).toEqual('$ 123,456.789');
    });
});
describe('formatCurrency', function () {
    it('returns formatted number with only defaults', function () {
        expect(new translator_1.Translator({ locale: locale }).formatCurrency(123456.789)).toEqual('123,456.789 $');
    });
    it('returns formatted number without any options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatCurrency(123456.789)).toEqual('123,456.789 $');
    });
    it('returns formatted number with options in messages', function () {
        expect(new translator_1.Translator({ messages: messages.setIn(['en', 'formats', 'number', 'currency', 'unit'], '€'), locale: locale }).formatCurrency(123456.789)).toEqual('123,456.789 €');
    });
    it('returns formatted number with override options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatCurrency(123456.789, { template: '€ %n' })).toEqual('€ 123,456.789');
    });
});
describe('formatPercentage', function () {
    it('returns formatted number with only defaults', function () {
        expect(new translator_1.Translator({ locale: locale }).formatPercentage(123456.789)).toEqual('123,456.789 %');
    });
    it('returns formatted number without any options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatPercentage(123456.789)).toEqual('123,456.789 %');
    });
    it('returns formatted number with options in messages', function () {
        expect(new translator_1.Translator({ messages: messages.setIn(['en', 'formats', 'number', 'percentage', 'template'], '% %n'), locale: locale }).formatPercentage(123456.789)).toEqual('% 123,456.789');
    });
    it('returns formatted number with override options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).formatPercentage(123456.789, { template: '% %n' })).toEqual('% 123,456.789');
    });
});
describe('interpolation', function () {
    it('detects interpolation', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).__hasInterpolation('this has interpolation %{foo}')).toBeTruthy();
    });
    it('detects interpolation (key must be present)', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).__hasInterpolation('this is not interpolation %{}')).toBeFalsy();
    });
    it('returns interpolated string when no translations is present', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Not translated yet %{status}', { status: 'interpolated' })).toEqual('Not translated yet interpolated');
    });
    it('returns translated interpolated string', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('Interpolated string', { foo: 'bar' })).toEqual('This was interpolated: bar');
    });
    it('handles combination of interpolation dictionary and message options', function () {
        expect(new translator_1.Translator({ messages: messages, locale: locale }).msg('scopedTransInterpolation', { scope: 'deep', foo: 'BAR!' })).toEqual('BAR! (again)');
    });
});
describe('connector to translation server', function () {
    var createSpace = function (overrides) {
        if (overrides === void 0) { overrides = {}; }
        var dispatch = jest.fn();
        var connector = new Connector_1.default({
            sync: true,
            apiUrl: 'http://translations.blueberry.io',
            apiToken: 'XYZ',
        }, dispatch);
        var translator = new translator_1.Translator(__assign({ messages: messages, locale: locale }, overrides), connector);
        return {
            dispatch: dispatch,
            connector: connector,
            translator: translator,
        };
    };
    it('should return default translation in connector', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('Something nice')).toEqual('Something nice');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.Something nice': {
                    data_type: 'string',
                    key: 'en.Something nice',
                    text: 'Something nice',
                },
            },
        });
    });
    it('should return undefined when disabledDefault', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('Not translated', { disableDefault: true })).toEqual(undefined);
        expect(connector.translationStore.toJS()).toEqual({});
    });
    it('should return text from messages', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('Homepage headline')).toEqual('Super headline');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.Homepage headline': {
                    data_type: 'string',
                    key: 'en.Homepage headline',
                    text: 'Super headline',
                },
            },
        });
    });
    it('returns interpolated string when no translations is present', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('Not translated yet %{status}', { status: 'interpolated' })).toEqual('Not translated yet interpolated');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.Not translated yet %{status}': {
                    data_type: 'string',
                    key: 'en.Not translated yet %{status}',
                    text: 'Not translated yet %{status}',
                },
            },
        });
    });
    it('should return text from messages with nested array key', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg(['homepage', 'Homepage headline'])).toEqual('Super headline under scope');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.homepage.Homepage headline': {
                    data_type: 'string',
                    key: 'en.homepage.Homepage headline',
                    text: 'Super headline under scope',
                },
            },
        });
    });
    it('should return deep object from messages', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('deep')).toEqual(messages.getIn(['en', 'deep']).toJS());
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.deep.scopedTransInterpolation': {
                    data_type: 'string',
                    key: 'en.deep.scopedTransInterpolation',
                    text: '%{foo} (again)',
                },
                'en.deep.scope.Homepage headline': {
                    data_type: 'string',
                    key: 'en.deep.scope.Homepage headline',
                    text: 'Super headline under deep scope',
                },
            },
        });
    });
    it('should return correct data type', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('formats.number.default.precision')).toEqual(10);
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.formats.number.default.precision': {
                    data_type: 'float',
                    key: 'en.formats.number.default.precision',
                    text: 10,
                },
            },
        });
    });
    it('should return text from first found key', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg(['deep.scope.Homepage headline', 'homepage.Homepage headline'])).toEqual('Super headline under deep scope');
        expect(translator.msg(['deep.scope.unknown', 'homepage.Homepage headline'])).toEqual('Super headline under scope');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.deep.scope.Homepage headline': {
                    data_type: 'string',
                    key: 'en.deep.scope.Homepage headline',
                    text: 'Super headline under deep scope',
                },
                'en.homepage.Homepage headline': {
                    data_type: 'string',
                    key: 'en.homepage.Homepage headline',
                    text: 'Super headline under scope',
                },
            },
        });
    });
    it('should return fallback text in default en from messages', function () {
        var _a = createSpace({ locale: 'es', fallbackLocale: 'en' }), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('Homepage headline', { scope: 'homepage' })).toEqual('Super headline under scope');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'es.homepage.Homepage headline': {
                    data_type: 'string',
                    key: 'es.homepage.Homepage headline',
                    text: 'Super headline under scope',
                },
            },
        });
    });
    it('should return default text when not found in locale and defaultLocale', function () {
        var _a = createSpace({ locale: 'es', fallbackLocale: 'en' }), translator = _a.translator, connector = _a.connector;
        expect(translator.msg('Not translated')).toEqual('Not translated');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'es.Not translated': {
                    data_type: 'string',
                    key: 'es.Not translated',
                    text: 'Not translated',
                },
            },
        });
    });
    it('Using array as keys for fallbacking should return default text from first not found key', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg(['deep.scope.unknown', 'homepage.unknown'])).toEqual('deep.scope.unknown');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.deep.scope.unknown': {
                    data_type: 'string',
                    key: 'en.deep.scope.unknown',
                    text: 'deep.scope.unknown',
                },
            },
        });
    });
    it('Using array as keys for fallbacking should return default text from first not found key with scope', function () {
        var _a = createSpace(), translator = _a.translator, connector = _a.connector;
        expect(translator.msg(['scope.unknown', 'homepage.unknown'], { scope: 'deep' })).toEqual('scope.unknown');
        expect(connector.translationStore.toJS()).toEqual({
            '/': {
                'en.deep.scope.unknown': {
                    data_type: 'string',
                    key: 'en.deep.scope.unknown',
                    text: 'scope.unknown',
                },
            },
        });
    });
});
describe('creating proper JSON structure from flatmap', function () {
    it('should create structured JSON', function () {
        var translator = new translator_1.Translator({ messages: flatMapMessages, locale: locale });
        expect(translator.__messages().getIn(['en', 'views', 'homepage', 'title'])).toEqual('Welcome!');
    });
});
describe('using messages from flatmaps', function () {
    it('finds correct key in messages', function () {
        expect(new translator_1.Translator({ messages: flatMapMessages, locale: locale }).msg('views.homepage.title')).toEqual('Welcome!');
    });
    it('finds correct key in messages for Spanish', function () {
        expect(new translator_1.Translator({ messages: flatMapMessages, locale: 'es' }).msg('views.homepage.title')).toEqual('Bueno!');
    });
    it('should combine keys into one string and find it in flatmap', function () {
        expect(new translator_1.Translator({ messages: flatMapMessages, locale: locale }).msg(['views', 'homepage', 'title'])).toEqual('Welcome!');
    });
    it('should be able to combine flatmap and structured messages', function () {
        expect(new translator_1.Translator({ messages: hybridMapMessages, locale: locale }).msg('views.homepage.title')).toEqual('Welcome!');
    });
    it('should be able to combine flatmap and structured messages for Spanish', function () {
        expect(new translator_1.Translator({ messages: hybridMapMessages, locale: 'es' }).msg('views.homepage.title')).toEqual('Bueno!');
    });
});
//# sourceMappingURL=translator.spec.js.map